/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */
/*
 * Copyright (C) 2024 Inochi Amaoto <inochiama@outlook.com>
 *
 * This file is generated from vendor pinout definition.
 */

#ifndef _DT_BINDINGS_PINCTRL_SG2002_H
#define _DT_BINDINGS_PINCTRL_SG2002_H

#include <dt-bindings/pinctrl/pinctrl-cv18xx.h>

#define PIN_AUD_AINL_MIC		2
#define PIN_AUD_AOUTR			4
#define PIN_SD0_CLK			6
#define PIN_SD0_CMD			7
#define PIN_SD0_D0			8
#define PIN_SD0_D1			10
#define PIN_SD0_D2			11
#define PIN_SD0_D3			12
#define PIN_SD0_CD			14
#define PIN_SD0_PWR_EN			15
#define PIN_SPK_EN			17
#define PIN_UART0_TX			18
#define PIN_UART0_RX			19
#define PIN_EMMC_DAT2			20
#define PIN_EMMC_CLK			21
#define PIN_EMMC_DAT0			22
#define PIN_EMMC_DAT3			23
#define PIN_EMMC_CMD			24
#define PIN_EMMC_DAT1			25
#define PIN_JTAG_CPU_TMS		26
#define PIN_JTAG_CPU_TCK		27
#define PIN_IIC0_SCL			28
#define PIN_IIC0_SDA			29
#define PIN_AUX0			30
#define PIN_GPIO_ZQ			35
#define PIN_PWR_VBAT_DET		38
#define PIN_PWR_RSTN			39
#define PIN_PWR_SEQ1			40
#define PIN_PWR_SEQ2			41
#define PIN_PWR_WAKEUP0			43
#define PIN_PWR_BUTTON1			44
#define PIN_XTAL_XIN			45
#define PIN_PWR_GPIO0			47
#define PIN_PWR_GPIO1			48
#define PIN_PWR_GPIO2			49
#define PIN_SD1_D3			51
#define PIN_SD1_D2			52
#define PIN_SD1_D1			53
#define PIN_SD1_D0			54
#define PIN_SD1_CMD			55
#define PIN_SD1_CLK			56
#define PIN_PWM0_BUCK			58
#define PIN_ADC1			59
#define PIN_USB_VBUS_DET		60
#define PIN_ETH_TXP			62
#define PIN_ETH_TXM			63
#define PIN_ETH_RXP			64
#define PIN_ETH_RXM			65
#define PIN_GPIO_RTX			67
#define PIN_MIPIRX4N			72
#define PIN_MIPIRX4P			73
#define PIN_MIPIRX3N			74
#define PIN_MIPIRX3P			75
#define PIN_MIPIRX2N			76
#define PIN_MIPIRX2P			77
#define PIN_MIPIRX1N			78
#define PIN_MIPIRX1P			79
#define PIN_MIPIRX0N			80
#define PIN_MIPIRX0P			81
#define PIN_MIPI_TXM2			83
#define PIN_MIPI_TXP2			84
#define PIN_MIPI_TXM1			85
#define PIN_MIPI_TXP1			86
#define PIN_MIPI_TXM0			87
#define PIN_MIPI_TXP0			88

#endif /* _DT_BINDINGS_PINCTRL_SG2002_H */
